#pragma once

#include <vector>
#include <list>
#include "TransScriptParser.h"

using namespace std;

#define MAX_TEXT_LENGTH 1024


//////////////////////////////////////////////////////////////////////////
class CTransCommand
{
	friend class CHookPoint;

private:
	int		m_nTransMethod;
	BOOL	m_bUnicode;
	BOOL	m_bAllSameText;
	BOOL	m_bClipKor;
	BOOL	m_bClipJpn;
	BOOL	m_bRemoveSpace;
	BOOL	m_bTwoByte;
	BOOL	m_bSafe;

	CString m_strArgScript;

	CTransCommand() : 
		m_nTransMethod(0), m_bUnicode(FALSE), m_bAllSameText(FALSE), m_bClipKor(FALSE), 
		m_bClipJpn(FALSE), m_bRemoveSpace(FALSE), m_bTwoByte(FALSE), m_bSafe(FALSE)
	{
		m_strArgScript.Empty();
		ZeroMemory(m_pOrigTextBuf, MAX_TEXT_LENGTH*2);
		ZeroMemory(m_pTransTextBuf, MAX_TEXT_LENGTH*2);
	};
	~CTransCommand() {};
public:
	BYTE	m_pOrigTextBuf[MAX_TEXT_LENGTH*2];
	BYTE	m_pTransTextBuf[MAX_TEXT_LENGTH*2];

	//void SetDistFromEBP(int nDistFromESP){ m_nDistFromESP = nDistFromESP; };
	void SetArgScript(LPCTSTR strScript){ m_strArgScript = strScript; };
	void SetTransMethod(int nMethod){ m_nTransMethod = nMethod; };
	void SetUnicode(BOOL bValue){ m_bUnicode = bValue; };
	void SetAllSameText(BOOL bAllSameText){ m_bAllSameText = bAllSameText; };
	void SetClipKor(BOOL bClipKor){ m_bClipKor = bClipKor; };
	void SetClipJpn(BOOL bClipJpn){ m_bClipJpn = bClipJpn; };
	void SetRemoveSpace(BOOL bRemove){ m_bRemoveSpace = bRemove; };
	void SetTwoByte(BOOL bEnable){ m_bTwoByte = bEnable; };
	void SetSafe(BOOL bEnable){ m_bSafe = bEnable; };

	//int	 GetDistFromEBP(){ return m_nDistFromESP; };
	CString	GetArgScript(){ return m_strArgScript; };
	int  GetTransMethod(){ return m_nTransMethod; };
	BOOL GetUnicode(){ return m_bUnicode; };
	BOOL GetAllSameText(){ return m_bAllSameText; };
	BOOL GetClipKor(){ return m_bClipKor; };
	BOOL GetClipJpn(){ return m_bClipJpn; };
	BOOL GetRemoveSpace(){ return m_bRemoveSpace; };
	BOOL GetTwoByte(){ return m_bTwoByte; };
	BOOL GetSafe(){ return m_bSafe; };
};




//////////////////////////////////////////////////////////////////////////
class CHookPoint
{
private:
	HMODULE		m_hModule;
	UINT_PTR	m_pCodePoint;
	CString		m_strModuleName;
	CTransScriptParser m_parser;
	vector<CTransCommand*> m_vectorTransCmd;

	CHookPoint();
	void ExecuteTransCmds(PREGISTER_ENTRY pRegisters);
	BOOL EncodeMultiBytes(LPSTR szKorean);
	BOOL EnforceTwoBtye(LPSTR szText);
	BOOL ReduceSpaceA(LPSTR szText, int nSizeTo);
	BOOL OverwriteTextBytes( LPVOID pDest, LPVOID pSrc, BOOL bWideChar );
	BOOL SearchTextA(UINT_PTR ptrBegin, LPCSTR cszText, list<LPVOID>* pTextList);
	BOOL SearchTextW(UINT_PTR ptrBegin, LPCWSTR cwszText, list<LPVOID>* pTextList);
	static void PointCallback(LPVOID pHookedPoint, PREGISTER_ENTRY pRegisters);

public:
	static CHookPoint* CreateInstance(CString strAddr);
	~CHookPoint();

	UINT_PTR		GetHookAddress();
	CString			GetHookAddressString();
	
	CTransCommand*	AddTransCmd(CString strArgScript);
	CTransCommand*	FindTransCmd(CString strArgScript);

	int				GetTransCmdCount();	
	CTransCommand*	GetTransCmd(int nIdx);
	void			DeleteTransCmd(int nIdx);
	void			DeleteAllTransCmd();
};
